package net.wanji.web.service.scheme.impl;

import net.wanji.web.common.constant.ControlModeConst;
import net.wanji.web.dto.LaneIdDTO;
import net.wanji.web.dto.LightIdDTO;
import net.wanji.web.dto.SaveSchemeConfigDTO;
import net.wanji.web.mapper.scheme.CrossLaneLightsMapper;
import net.wanji.web.mapper.scheme.CrossPhaseLightsMapper;
import net.wanji.web.mapper.scheme.CrossPhaseMapper;
import net.wanji.web.mapper.scheme.CrossSchemeMapper;
import net.wanji.web.mapper.scheme.LaneInfoMapper;
import net.wanji.web.po.scheme.CrossLaneLightsPO;
import net.wanji.web.po.scheme.CrossPhaseLightsPO;
import net.wanji.web.po.scheme.CrossPhasePO;
import net.wanji.web.po.scheme.CrossSchemePO;
import net.wanji.web.service.scheme.SchemeConfigService;
import net.wanji.web.vo.scheme.LaneIdsVO;
import net.wanji.web.vo.scheme.LightIdVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/1/5 10:27
 */
@Service
public class SchemeConfigServiceImpl implements SchemeConfigService {
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final CrossLaneLightsMapper crossLaneLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;

    public SchemeConfigServiceImpl(CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper, LaneInfoMapper laneInfoMapper, CrossLaneLightsMapper crossLaneLightsMapper, CrossPhaseLightsMapper crossPhaseLightsMapper) {
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.crossLaneLightsMapper = crossLaneLightsMapper;
        this.crossPhaseLightsMapper = crossPhaseLightsMapper;
    }

    @Override
    @Transactional
    public void saveSchemeConfig(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        Integer timeType = saveSchemeConfigDTO.getTimeType();
        if (timeType == 2) { //环式
            handleRingTime(saveSchemeConfigDTO);
        } else if (timeType == 1) { //阶段式
            // TODO: 2023/1/6 处理阶段式
        }
    }

    @Override
    public LightIdVO getLightByLane(LaneIdDTO laneIdDTO) {
        String crossId = laneIdDTO.getCrossId();
        String laneId = laneIdDTO.getLaneId();
        Integer lightId = crossLaneLightsMapper.selectLightsIdByLaneId(crossId, laneId);
        LightIdVO lightIdVO = new LightIdVO();
        lightIdVO.setCrossId(crossId);
        lightIdVO.setLightId(lightId);
        return lightIdVO;
    }

    @Override
    public LaneIdsVO getLanesByLight(LightIdDTO lightIdDTO) {
        String crossId = lightIdDTO.getCrossId();
        LaneIdsVO laneIdsVO = new LaneIdsVO();
        laneIdsVO.setCrossId(crossId);

        Integer lightId = lightIdDTO.getLightId();
        List<CrossLaneLightsPO> crossLaneLightsPOList = crossLaneLightsMapper.selectLanesByLightId(crossId, lightId);
        List<String> laneIds = crossLaneLightsPOList.stream()
                .map(CrossLaneLightsPO::getLaneId)
                .collect(Collectors.toList());
        laneIdsVO.setLaneIds(laneIds);

        return laneIdsVO;
    }

    private void handleRingTime(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        // 更新方案基础信息表
        updateCrossScheme(saveSchemeConfigDTO);
        // 更新相位基础表、相位灯组关系表
        updateCrossPhase(saveSchemeConfigDTO);
        // 更新协调相位ID
        updateCoordPhase(saveSchemeConfigDTO);
    }

    private void updateCoordPhase(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        List<SaveSchemeConfigDTO.TimeScheme> timeSchemeList = saveSchemeConfigDTO.getTimeSchemeList();
        for (SaveSchemeConfigDTO.TimeScheme timeScheme : timeSchemeList) {
            String schemeNo = timeScheme.getSchemeNo();
            Integer schemeId = crossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            String coordPhaseNo = timeScheme.getCoordPhaseNo();
            Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndPhaseNo(crossId, schemeId, coordPhaseNo);
            crossSchemeMapper.updateCoordPhase(crossId, schemeNo, phaseId);
        }
    }

    private void updateCrossPhase(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        crossPhaseMapper.deleteByCrossId(crossId);
        crossPhaseLightsMapper.deleteByCrossId(crossId);
        for (SaveSchemeConfigDTO.TimeScheme timeScheme : saveSchemeConfigDTO.getTimeSchemeList()) {
            String schemeNo = timeScheme.getSchemeNo();
            List<SaveSchemeConfigDTO.TimeScheme.TimeRingListElement> ringList = timeScheme.getRingList();
            for (SaveSchemeConfigDTO.TimeScheme.TimeRingListElement ring : ringList) {
                Integer ringNo = ring.getRingNo();
                List<SaveSchemeConfigDTO.TimeScheme.TimeRingListElement.TimePhaseListElement> phaseList = ring.getPhaseList();
                for (int i = 0; i < phaseList.size(); i++) {
                    SaveSchemeConfigDTO.TimeScheme.TimeRingListElement.TimePhaseListElement phase = phaseList.get(i);
                    String phaseNo = phase.getPhaseNo();
                    Integer yellowFlash = phase.getYellowFlash();
                    Integer redFlash = phase.getRedFlash();
                    Integer greenTime = phase.getGreenTime();
                    Integer greenFlashTime = phase.getGreenFlashTime();
                    Integer yellowTime = phase.getYellowTime();
                    Integer redTime = phase.getRedTime();
                    Integer minGreenTime = phase.getMinGreenTime();
                    Integer maxGreenTime = phase.getMaxGreenTime();
                    // 获取方案ID
                    Integer schemeId = crossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
                    CrossPhasePO crossPhasePO = new CrossPhasePO();
                    crossPhasePO.setPhaseNo(phaseNo);
                    crossPhasePO.setSort(i + 1);
                    crossPhasePO.setCrossId(crossId);
                    crossPhasePO.setPlanId(schemeId);
                    crossPhasePO.setRingNo(ringNo);
                    // 控制模式
                    if (yellowFlash == 0 && redFlash == 0) {
                        crossPhasePO.setControlMode(ControlModeConst.FIXED_CYCLE);
                    } else if (yellowFlash == 1) {
                        crossPhasePO.setControlMode(ControlModeConst.YELLOW_FLASH);
                    } else if (redFlash == 1) {
                        crossPhasePO.setControlMode(ControlModeConst.RED_FLASH);
                    }
                    crossPhasePO.setGreenTime(greenTime);
                    crossPhasePO.setGreenFlashTime(greenFlashTime);
                    crossPhasePO.setYellowTime(yellowTime);
                    crossPhasePO.setRedTime(redTime);
                    crossPhasePO.setMinGreenTime(minGreenTime);
                    crossPhasePO.setMaxGreenTime(maxGreenTime);
                    crossPhasePO.setPhaseTime(greenTime + greenFlashTime + yellowTime + redTime);
                    // 插入相位表记录，获取ID
                    crossPhaseMapper.insertOne(crossPhasePO);
                    Integer phaseId = crossPhasePO.getId();
                    // 更新相位灯组关系表
                    updateCrossPhaseLights(crossId, phase, phaseId);
                }
            }
        }
    }

    private void updateCrossPhaseLights(String crossId,
                                        SaveSchemeConfigDTO.TimeScheme.TimeRingListElement.TimePhaseListElement phase,
                                        Integer phaseId) {
        Set<CrossPhaseLightsPO> crossPhaseLightsPOSet = new HashSet<>();
        for (SaveSchemeConfigDTO.TimeScheme.TimeRingListElement.TimePhaseListElement.TimeLaneListElement lane : phase.getLaneList()) {
            CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
            Integer dir = lane.getDir();
            Integer turn = lane.getTurn();
            Integer category = lane.getCategory();
            // 根据车道获得灯组ID
            String laneId = laneInfoMapper.selectIdByCrossIdDirTurn(crossId, dir, turn, category);
            Integer lightsId = crossLaneLightsMapper.selectLightsIdByLaneId(crossId, laneId);
            crossPhaseLightsPO.setLightsId(lightsId);
            crossPhaseLightsPO.setPhaseId(phaseId);
            crossPhaseLightsPO.setCrossId(crossId);
            crossPhaseLightsPOSet.add(crossPhaseLightsPO);
        }
        crossPhaseLightsMapper.insertBatchBySet(crossPhaseLightsPOSet);
    }

    private void updateCrossScheme(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        crossSchemeMapper.deleteByCrossId(crossId);
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        List<CrossSchemePO> crossSchemePOList = new ArrayList<>();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            CrossSchemePO crossSchemePO = new CrossSchemePO();
            String schemeNo = phaseScheme.getSchemeNo();// 方案号
            crossSchemePO.setSchemeNo(schemeNo);
            String name = phaseScheme.getName(); // 方案名称
            crossSchemePO.setName(name);
            crossSchemePO.setCrossId(crossId);
            // 获取方案周期、协调相位ID、相位差
            List<SaveSchemeConfigDTO.TimeScheme> timeSchemeList = saveSchemeConfigDTO.getTimeSchemeList();
            for (SaveSchemeConfigDTO.TimeScheme timeScheme : timeSchemeList) {
                String schemeNoInTime = timeScheme.getSchemeNo();
                if (Objects.equals(schemeNo, schemeNoInTime)) {
                    crossSchemePO.setCycle(timeScheme.getCycle());
                    crossSchemePO.setOffset(timeScheme.getOffset());
                }
            }
            crossSchemePO.setSource(2); // 来自平台
            crossSchemePO.setIsDeleted(0);
            crossSchemePOList.add(crossSchemePO);
        }
        crossSchemeMapper.insertBatch(crossSchemePOList);
    }
}
